import { SearchParams, SearchResult } from "server/models/monolyth"
import { Component } from "vue";
export const DefaultActions = {
    EDIT:{id:'edit', label:'Editar', icon:'bi-pencil-square' } as ItemAction,
    ADD:{id:'add', label:'Añadir', icon:'bi-plus-square' } as ItemAction,
    DELETE:{id:'delete', label:'Eliminar', icon:'bi-x-square' } as ItemAction,
    VIEW:{id:'view', label:'Ver', icon:'bi-folder2-open' } as ItemAction,

}
export interface ItemAction{
    id:string;
    label:string;
    icon?:string
}

export interface Filter{
    /**
     * Tipo de dato del filtro, determina el componente de formulario que se mostrará
     */
    type:'text'|'boolean'|'array';
    /**
     * Nombre del atributo que se mandará a la búsqueda
     */
    field:string;
    /**
     * Etiqueta que se mostrará encima del control
     */
    label:string
    /**
     * Valor del campo
     */
    value?:any;
    /**
     * Datos adicionales que pueda contener (por ejemplo, los valores disponbles de un select)
     */
    data?:any
}
export interface SectionHandler<T> {
    load(id:any):Promise<T>;
    newEntity():T;
    getId():string;
    getName():string;
    getResults(filters:Record<string,Filter>,page:number,records:number): Promise<SearchResult<T>>;
    getFilters(): Record<string,Filter>;
    onItemActionPerformed(action:ItemAction,item:T):Promise<any>;
    getItemActions():ItemAction[];
    saveEntity(entity:T):Promise<void>;
    getListRenderer():Component;
    getEntityEditor():Component;
    getEntityView():Component;
}

