import router from "@/router";
import { ErrorState, useStore } from "@/store";
import { GameSectionHandler } from "./games";
import { InstanceSectionHandler } from "./instances";
import { SectionHandler } from "./sections";
import { UserSectionHandler } from "./users";

const store = useStore();
const sectionCache : Record<string,SectionHandler<any>> = {};

export function getSectionHandler(name:string):SectionHandler<any>{
    if(name == 'users'){
        if(sectionCache[name] === undefined){
            sectionCache[name] = new UserSectionHandler();
        }
    }else if(name == 'games'){
        if(sectionCache[name] === undefined){
            sectionCache[name] = new GameSectionHandler();
        }
    }else if(name == 'instances'){
        if(sectionCache[name] === undefined){
            sectionCache[name] = new InstanceSectionHandler();
        }
    }

    return sectionCache[name];
}

/**
 * Muestra un error en la ventana modal informando del problema.
 * @param message 
 */
export function systemError(message:string){
    store.commit('setError',{
        visible:true,
        title:'Error',
        message
    } as ErrorState);
}
/*
export function checkAccess(right:string):void{
    const user = store.state.user;
    const access = (user != null) && user.privileges.indexOf(right) != -1;
    if(!access){
        router.push({path:'/error'})
    }
}*/