/**
 * Interfaz de una función de validación
 */
export interface Validator{
    (field:string):string|null;
}

export enum Mode {
    Create,
    Update
}

export interface ValidationStatus{
    valid:boolean;
    cause:string|null;
}

export interface EditableModel<T>{
    model:T|null;
    mode:Mode;
    status:ValidationStatus;
}

export function validMail(mail:string):boolean{
    return (/^[-._a-z0-9]+@[-_a-z0-9]+\.[a-z]{2,3}/.test(mail.toLowerCase())) == true;
}
/**
 * Devuelve una cadena de error si el email indicado no es válido
 * @param mail Cadena con el email
 * @returns Un string con el error o null si es válido
 */
 export function validateEmail(mail:string|null):ValidationStatus{
    let status:ValidationStatus = {valid:true,cause:null};
    if(!mail){
        status = {valid:false,cause:'El correo electrónico no puede estar vacío'}
    }else if(!validMail(mail)){
        status = {valid:false,cause:'El correo no es válido'}
    }
    return status;
}


export function nonEmptyString(field:string,value:string|null|undefined):ValidationStatus{
    let status:ValidationStatus = {valid:true,cause:null};
    if(value == undefined || value == null || value == ''){
        status = {valid:false,cause:`El ${field} no puede estar vacío`}
    }

    return status;
}

export function validatePassword(p1:string|null,p2:string|null):ValidationStatus{
    let status:ValidationStatus = {valid:true,cause:null};
    if(!p1){
        status = {valid:false,cause:'El password está vacío'}
    }else if(p1 != p2){
        status = {valid:false,cause:'Los passwords no coinciden'}
    }else if(p1.length < 8){
        status = {valid:false,cause:'El password debe tener al menos 8 caracteres'};
    }
    return status;
}
