import { Game, Link, Privileges, User, WithToken } from 'server/models/monolyth';
import { Component } from 'vue';
import { createStore, Store } from 'vuex'
import VuexPersistence from 'vuex-persist';

export type ModalStateButton = 'accept'|'cancel'|'close';
export interface ErrorState{
  visible:boolean;
  title?:string;
  message?:string;
}
export interface ModalState{
  /**
   * Se permite cierto grado de control sobre el tamaño, si se prevee
   * que el componente tenga poco contenido, un tamaño pequeño hará
   * que quede más compacto y adecuado.
   */
  size?:'modal-sm'|'modal-lg'|'modal-xl',
  /**
   * Determina si el modal será visible
   */
  visible:boolean;
  /**
   * Título de la ventana
   */
  title?:string;
  /**
   * Botones
   */
  buttons?:ModalStateButton[];
  /**
   * Componente que se pintará dentro del modal
   */
  component?:Component;
  /**
   * Modelo de datos que se pasará al componente
   */
  data?:any;
  /**
   * Callback que se usará al pulsar el botón de ok
   * @param newdata Mutaciones aplicadas sobre el estado original.
   */
  callback?(newdata:any):void;
}


/**
 * Persistencia en el navegador
 */
 const vuexLocal = new VuexPersistence<BackofficeStore>({
  storage: window.localStorage,
  key:'vuex_backoffice'
});

export interface BackofficeStore{
  modal:ModalState;
  error:ErrorState;
  user:WithToken<User>|null;
  editedGame?:Game;
}

const store = createStore<BackofficeStore>({
  state: {
    modal:{visible:false,title:'',buttons:[]},
    error:{visible:false,title:'',message:''},
    user:null
  },
  getters: {
  },
  mutations: {
    setModal(store:BackofficeStore,modalState:ModalState){
      store.modal = {...modalState};
    },
    setEditedGame(store:BackofficeStore,game:Game){
      store.editedGame = game;
    },
    setError(store:BackofficeStore,error:ErrorState){
      store.error = error;
    },
    setUser(store:BackofficeStore,user:WithToken<User>|null){
      store.user = user;
    }
  },
  actions: {
  },
  modules: {
  },
  plugins:[vuexLocal.plugin]
})

export function useStore():Store<BackofficeStore> {
  return store;
}